#!/bin/bash
#Autor: José Antonio Serrano Rosso
#Actualizado: 08-01-2013
#Descripción: Instalador de SAWESO

#Obtenemos la fecha en formatos dd-mm-yyyy y la hora en formato hh:mm:ss
FECHA="$(date +"%d-%m-%Y")"
HORA="$(date +"%H_%M_%S")"

#Formateo de la salida por pantalla del instalador
txtrst=$(tput sgr0) # Text reset
txtred=$(tput setaf 1) # Red

#Directorios de trabajo
DIR_TEMPLATES="./saweso_templates"

#Variables
USER_SAWESO=tomcat6

####################################### REQUISITOS PREVIOS #######################################
#La instalación del sistema SAWESO ha sido comprobada bajo el Sistema Operativo Debian Squeeze (6.0.6).

##Instalar la paquetería necesaria , desde una terminal ejecutamos:
echo "${txtred} Comprobamos la instalanción de la paquetería necesaria ... ${txtrst}"
apt-get -y install openjdk-6-jre openjdk-6-jdk
apt-get -y install tomcat6 tomcat6-docs tomcat6-examples tomcat6-admin
apt-get -y install mysql-server
apt-get -y install postfix
apt-get -y install ssh autossh
apt-get -y install netcat-openbsd

##Establecemos la contraseña de root al servidor MySQL (por ejemplo: "holamundo")
echo "${txtred} Establecemos la contraseña de root (holamundo) al servidor MySQL ${txtrst}"
mysqladmin -h localhost -u root password "holamundo" 2> /dev/null


####################################### SCRIPT DE INSTALACION #######################################
#Comprobaciones iniciales
usuario=`whoami`
if [ "$usuario" != "root" ];then
	echo "FAIL: El script $0 debe ser ejecutado por el usuario root"
	exit 1
fi


#Instalacion de saweso
if [ -d $DIR_TEMPLATES ];then
	#Limpiamos instalaciones de saweso anteriores
	if [ -d /etc/saweso/PlantillasEjecucion ] && [ -d /var/log/saweso ];then
		echo "${txtred} Limpiando datos de instalaciones anteriores ... (¡Se guardará una backup en /var/backups/saweso.backup.$FECHA.$HORA!) ${txtrst}"
		mkdir -p /var/backups/saweso.backup.$FECHA.$HORA/etc.saweso.PlantillasEjecucion
		mkdir -p /var/backups/saweso.backup.$FECHA.$HORA/var.log.saweso
		cp -a /etc/saweso/PlantillasEjecucion /var/backups/saweso.backup.$FECHA.$HORA/etc.saweso.PlantillasEjecucion
		rm -rf /etc/saweso/PlantillasEjecucion
		cp -a /var/log/saweso /var/backups/saweso.backup.$FECHA.$HORA/var.log.saweso
		rm -rf /var/log/saweso
	fi
	#Instalacion de ficheros y directorios
	echo "${txtred} Copiando directorio de scripts a /etc/saweso ... ${txtrst}"
	mkdir -p /etc/saweso
	cp -a $DIR_TEMPLATES/etc.saweso/* /etc/saweso
	echo "${txtred} Copiando script saweso.sh a /usr/bin ... ${txtrst}"
	cp -a $DIR_TEMPLATES/usr.bin/saweso.sh /usr/bin
	echo "${txtred} Creando directorio de log /var/log/saweso ... ${txtrst}"
	mkdir -p /var/log/saweso
	#Añadimos un usuario administrador (por ejemplo: “admin”, “admin”) al fichero de configuración de usuarios de Apache Tomcat
	/etc/init.d/tomcat6 stop
	cp $DIR_TEMPLATES/etc.tomcat6/tomcat-users.xml /etc/tomcat6/
	chown root:tomcat6 /etc/tomcat6/tomcat-users.xml
	chmod 640 /etc/tomcat6/tomcat-users.xml
	/etc/init.d/tomcat6 start
	#Establecemos permisos correctos para el usuario y grupo que ejecutará el sistema
	chown -R $USER_SAWESO:$USER_SAWESO /etc/saweso
	chown $USER_SAWESO:$USER_SAWESO /usr/bin/saweso.sh
	chmod 755 /usr/bin/saweso.sh
	chown -R $USER_SAWESO:$USER_SAWESO /var/log/saweso

	#Creacion y carga de la base de datos
	PASSBD=""
	TESTBD=1
	until [ -n "$PASSBD" ] && [ $TESTBD -eq 0 ]; do
		echo -e "\nIntroduzca la contraseña de root de MySQL:"
		read PASSBD
		mysql -u"root" -p"$PASSBD" -D "mysql" -e "use mysql;" -N
		if [ $? -eq 0 ];then
			TESTBD=0
		fi
	done
	
	echo "${txtred} Actualizamos la contraseña de la BD en el fichero de configuración (/etc/saweso/saweso.conf) ${txtrst}"
	sed -i -e 's/'holamundo'/'$PASSBD'/' /etc/saweso/saweso.conf

	echo "${txtred} Borrando BD saweso existente ... ${txtrst}"
	mysqladmin -uroot -p$PASSBD drop saweso

	echo "${txtred} Creando nueva BD saweso ... ${txtrst}"
	mysqladmin -uroot -p$PASSBD create saweso

	echo "${txtred} Cargando estructura de tablas ... ${txtrst}"
	mysql -u root -p$PASSBD saweso < $DIR_TEMPLATES/BD/tablas_BD.txt

	echo "${txtred} Cargando datos iniciales en las tablas ... ${txtrst}"
	mysql -u root -p$PASSBD saweso < $DIR_TEMPLATES/BD/datos_iniciales_BD.txt
else
	echo "FAIL: no existe el directorio de templates $DIR_TEMPLATES en la ruta donde se está ejecutando $0"
fi


####################################### REQUISITOS MANUALES #######################################
# Copiar el certificado id_dsa en el directorio /etc/saweso
# Borrar clave del certificado para evitar que la solicite en cada conexion
## ssh-keygen -p -P "<CLAVE>" -N "" -f /etc/saweso/id_dsa
## chown -R $USER_SAWESO:$USER_SAWESO /etc/saweso
## chmod 600 /etc/saweso/id_dsa

# Copiar certificado id_rsa en el directorio /etc/saweso
# Borrar clave del certificado para evitar que la solicite en cada conexion
## ssh-keygen -p -P "<CLAVE>" -N "" -f /etc/saweso/id_rsa
## chown -R $USER_SAWESO:$USER_SAWESO /etc/saweso
## chmod 600 /etc/saweso/id_rsa

# Revisar el fichero de configuracion general (/etc/saweso/saweso.conf) para comprobar que los nombres de los certificados coinciden
