#!/bin/bash
#Autor: <Nombre desarrollador>
#Actualizado: <fecha actualización>
#Descripcion: <Descripcion del script>

#Obtenemos la fecha en formatos dd-mm-yyyy y la hora en formato hh:mm:ss
FECHA="$(date +"%d-%m-%Y")"
HORA="$(date +"%H_%M_%S")"

#Variables
declare -a ADJUNTOS
EJECUCION=$1
IP=$2

#Ficheros
ENCABEZADO=$EJECUCION.$FECHA.$HORA.$IP			#Encabezado para todos los ficheros
LOG=$ENCABEZADO.saweso.log				#Log de ejecucion en el equipo final
RESULTADOS=$ENCABEZADO.tar.gz				#Archivado que se creará para transportar los resultados
SALIDA=salida.log					#Si se redirecciona la salida de algún comando a salida.log, esta se incluirá en la notificación por e-mail

#Directorios de trabajo
DIR_REGISTRO="/tmp"  					#Directorio de trabajo
DIR_RESULTADOS="$DIR_REGISTRO/$EJECUCION/$FECHA.$HORA"	#Directorio para almacenar los resultados de la ejecucion


############################################################# DEFINICION DE FUNCIONES ############################################################# 
#NOMBRE: ComprobarEntornoEjecucion - Comprueba el entorno en el host de destino para asegurar la correcta ejecución del script
#SINOPSIS: ComprobarEntornoEjecucion
#SALIDA: produce la salida con errores (exit 1) si encuentra algún parámetro que no permita la ejecucion del script
ComprobarEntornoEjecucion()
{
if [ -z $EJECUCION ] && [ -z $IP ];then
	logger "SAWESO.SH: No se ha especificado código de incidencia o dirección ip del equipo"
        exit 1
elif [ ! -d $DIR_REGISTRO ] && [ ! -r $DIR_REGISTRO ] && [ ! -w "$DIR_REGISTRO" ] && [ ! -x "$DIR_REGISTRO" ];then
	logger "SAWESO.SH: No se puede escribir en el directorio de registro $DIR_REGISTRO"
        exit 1
else
	logger "SAWESO.SH: Ejecución de saweso (Ejecucion: $EJECUCION; DIR_RESULTADOS: $DIR_RESULTADOS)"
fi
}


#NOMBRE: log - Registra la actividad de la ejecucion del script en el directorio de registro
#SINOPSIS: log <mensaje de texto>
#SALIDA: produce la salida con errores (exit 1) sino puede registrar el mensaje
log()
{
mensaje=$1
encabezado="`date +"%d-%m-%Y"` `date +"%T"`"


if [ -n "$EJECUCION" ] && [ -n "$EJECUCION" ]; then
        if [ -d "$DIR_REGISTRO/$EJECUCION" ] && [ -r "$DIR_REGISTRO/$EJECUCION" ]; then
		mkdir -p $DIR_RESULTADOS 2> /dev/null
		chmod 700 $DIR_RESULTADOS
                echo "$encabezado: $mensaje" >> $DIR_RESULTADOS/$LOG
        elif [ -d "$DIR_REGISTRO" ] && [ -r "$DIR_REGISTRO" ];then
                mkdir -p $DIR_REGISTRO/$EJECUCION 2> /dev/null
                chmod 700 $DIR_REGISTRO/$EJECUCION
		mkdir -p $DIR_RESULTADOS 2> /dev/null
                chmod 700 $DIR_RESULTADOS
                echo "$encabezado: $mensaje" >> $DIR_RESULTADOS/$LOG
        else
                logger "SAWESO.SH: No se ha podido registrar el mensaje $mensaje. No existe el directorio de log $DIR_REGISTRO"
                exit 1
        fi
else
        logger "SAWESO.SH: No se ha podido registrar el mensaje en el fichero de log. Falta código de incidencia o el mensaje esta vacío"
        exit 1
fi
}


#NOMBRE: Resultados - empaqueta los resultados en .tar.gz para su posterior transporte hacia el sistema saweso
#SINOPSIS: Resultados
Resultados()
{
cd $DIR_RESULTADOS
tar -cpzf $RESULTADOS * 2> /dev/null
mv $RESULTADOS $DIR_REGISTRO/$EJECUCION
}


############################################################# FUNCIONES MODIFICABLES #############################################################
#NOMBRE: Precondicion - Especifica la condicion que debe cumplirse para que se proceda a ejecutar el script (funcion Script)
#SINOPSIS: Precondicion
#SALIDA: devuelve 1 - Cumple la condicion; 0 - No cumple la condicion
Precondicion()
{
#EJEMPLO DE PRECONDICION. MODIFICAR SEGUN NECESIDADES 
#Solo se ejecutará la funcion Script si estamos en un servidor de seguridad (f0) bajo Debian Etch (4.0) 
coincidencia=`hostname -f | awk -F"." '{print $1}'` 

if [ "$coincidencia" == "f0" ];then 
        distro=`cat /etc/debian_version` 
        if [ "$distro" == "4.0" ];then 
                return 1 
        else 
                return 0 
        fi 
else 
        return 0 
fi 
}


#NOMBRE: Script - comando, script o workaorund a ejecutar en el equipo final
#SINOPSIS: Script
#SALIDA: si deseamos redireccionar la salida o salidas a ficheros de texto debemos tener en cuenta que:
#- El fichero de salida debe almacenarse en $DIR_RESULTADOS
#- El nombre del fichero debe comenzar con la cadena "$ENCABEZADO"
#- Si deseamos ademas que la salida se adjunte en el informe mediante correo electrónico, el nombre del fichero deberá finalizar en "$SALIDA"
#Ejemplo: uname -a >> $DIR_RESULTADOS/$ENCABEZADO.uname.$SALIDA
#- Si queremos utilizar ficheros adjuntos que transportamos junto a la plantilla de ejecucion debemos considerar que los ficheros estarán disponibles en la ruta $DIR_REGISTRO
Script()
{
#EJEMPLO DE SCRIPT. MODIFICAR SEGUN NECESIDADES 
#Copiamos los ficheros adjuntos que transporta la plantilla al directorio /var/www
cp -a $DIR_REGISTRO/fichero_adjunto1 /var/www/
if [ "$?" == "0"  ];then
	echo -e "TEST OK (`hostname -f`): Fichero fichero_adjunto1 copiado correctamente a /var/www" >> $DIR_RESULTADOS/$ENCABEZADO.copia_ficheros.$SALIDA 
else
	echo -e "TEST FAIL (`hostname -f`): Fallo al copiar el fichero fichero_adjunto1 a /var/www" >> $DIR_RESULTADOS/$ENCABEZADO.copia_ficheros.$SALIDA 
fi

cp -a $DIR_REGISTRO/fichero_adjunto2 /var/www/
if [ "$?" == "0"  ];then
	echo -e "TEST OK (`hostname -f`): Fichero fichero_adjunto2 copiado correctamente a /var/www" >> $DIR_RESULTADOS/$ENCABEZADO.copia_ficheros.$SALIDA 
else
	echo -e "TEST FAIL (`hostname -f`): Fallo al copiar el fichero fichero_adjunto2 a /var/www" >> $DIR_RESULTADOS/$ENCABEZADO.copia_ficheros.$SALIDA 
fi
}


#NOMBRE: Adjuntos - Carga el array ADJUNTOS con los comandos que producirán los ficheros adjuntos que se desean incorporar en los resultados tras la ejecución del script
#SINOPSIS: Adjuntos
#SALIDA: las salidas de los comandos que generarán los adjuntos deben cumprir las siguientes condiciones:
#- El fichero de salida debe almacenarse en $DIR_RESULTADOS
#- El nombre del fichero debe comenzar con la cadena "$ENCABEZADO"
#- Si deseamos ademas que la salida se adjunte en el informe mediante correo electrónico, el nombre del fichero deberá finalizar en "$SALIDA"
Adjuntos()
{
#EJEMPLO DE CAPTURA DE ADJUNTOS. MODIFICAR SEGUN NECESIDADES
#Recogemos la siguiente información del equipo 
lspci >> $DIR_RESULTADOS/$ENCABEZADO.lspci 
uname -ra >> $DIR_RESULTADOS/$ENCABEZADO.uname 
cp /var/log/syslog $DIR_RESULTADOS/$ENCABEZADO.syslog 

#Notificamos por email de la información recogida 
echo "Adjuntos capturados en el equipo: lspci, uname -ra y syslog" >> $DIR_RESULTADOS/$ENCABEZADO.script.$SALIDA 
}


######################################################## FIN FUNCIONES MODIFICABLES #############################################################
############################################################# CUERPO DEL SCRIPT #############################################################

ComprobarEntornoEjecucion
log "Entorno de ejecucion comprobado OK"
Precondicion
if [ $? -eq 1 ];then
	log "#################### INICIO EJECUCION $0 ($FECHA.$HORA) ####################"
        log "Precondicion para ejecucion de $0 OK"
	Script
	log "Script $0 ejecutado con exito"
	#¡¡DESCOMENTAR LAS SIGUIENTES LINEAS SI QUEREMOS UTILIZAR LA FUNCION DE ADJUNTOS!!	
	#Adjuntos
	#log "Se han recopilado los ficheros adjuntos solicitados"
	#log "*) Fichero saweso.log con el log de la ejecucion"
	#log "*) Ficheros adjuntos solicitados"
	#log "*) Fichero .txt con la salida del script $0"
	log "#################### FIN EJECUCION $0 ($FECHA.$HORA) ####################"
	Resultados
else
        log "#################### INICIO EJECUCION $0 ($FECHA.$HORA) ####################"
	log "Precondicion para ejecucion de $0 FAIL. No se ejecutará nada"
	log "#################### FIN EJECUCION $0 ($FECHA.$HORA) ####################"
	Resultados
	exit 1
fi
